#!usr/bin/python

import sys
import string
import copy

recursion_count = 0

# Parses out the sudoku games
def parse_games(file):
    games = []
    line = file.readline()
    while(line):
        # Lines above serve to throw out first line (game number)
        game = []
        # Add 9 rows
        for i in range(9):
            game.append(list(file.readline().rstrip()))
        games.append(game)
        line = file.readline()
    return games

# Provides the solution for a game structure passed into it
def solve_game(game):
    # First, convert the game into a possibility matrix
    matrix = conv_to_matrix(game)

    global recursion_count
    recursion_count = 0

    count = 0
    while(not solved(matrix)):

        matrix, changed = trim(matrix)

        if not changed:
            # Solution has converged; time to solve by guessing
            matrix = solve_by_guessing(matrix)
        
        if matrix == 0:
            print "No matrix found. Error...?"
            sys.exit()
        if consistent(matrix):
            pass
        else:
            print "Inconsistent!"
            sys.exit()
        count += 1


    print "Solved in %s moves." % count
    # Puzzle is now solved; convert back to original game format and return
    return conv_to_game(matrix)


# Take a given matrix and plug in solutions to try to solve it
def solve_by_guessing(matrix):

    global recursion_count
    recursion_count += 1

    # First, trim until converged
    changed = True
    while(changed):
        matrix, changed = trim(matrix)
    
    # Now that it's converged, is it solved?
    if solved(matrix) and consistent(matrix):
        recursion_count -= 1
        return matrix
    elif not consistent(matrix):
        recursion_count -= 1
        return 0
    
    # If it's not solved, then we need to make a guess

    # Find each collapsible list and collapse in every possible way
    new_matrix = copy.deepcopy(matrix)
    for row_no, row in enumerate(matrix):
        for col_no, block in enumerate(row):
            if len(block) > 1:
                for possibility in block:
                    # Try changing to each possibility
                    new_matrix[row_no][col_no] = [possibility]
                    # Recurse into a copy of this function with this guess made
                    sol_matrix = solve_by_guessing(copy.deepcopy(new_matrix))
                    # If a valid matrix was returned, then the matrix was solved
                    if sol_matrix:
                        recursion_count -= 1
                        return sol_matrix

    recursion_count -= 1
    return 0

def print_matrix(matrix):
    string = ''
    for row in matrix:
        for block in row:
            string += ''.join(map(str, block)).ljust(7)
        string += '\n'
    print string

# Converts a game grid with 0's to a matrix of possibilities for each block
def conv_to_matrix(game):
    matrix = []
    for row in game:
        row_to_add = []
        for block in row:
            if block == '0':
                row_to_add.append(range(1,10))
            else:
                row_to_add.append([int(block)])
        matrix.append(row_to_add)
    return matrix

# Converts a matrix of blocks into the original game grid format
def conv_to_game(matrix):
    game = []
    for row in matrix:
        game_row = []
        for block in row:
            game_row.append(block[0])
        game.append(game_row)
    return game

# Trims down possibilities by eliminating duplicates within
# rows, columns, and 3x3 boxes
def trim(matrix):
    # Only allow one alteration per pass to avoid race conditions
    changed = False
    new_matrix = []
    for row_no, row in enumerate(matrix):
        new_row = []
        # Look at each block in the row, and cull out values
        # that are already definitely in that row, column, or 3x3 box
        for col_no, block in enumerate(row):
            new_block = block
            # Just use the block if it's already the only choice
            if len(block) == 1:
                new_row.append(new_block)
            # Otherwise cull out values that aren't possibilities,
            # then use that modified block
            else:
                for val in block:
                   if not changed:
                        if val_in_row(matrix, row_no, val) \
                                or val_in_col(matrix, col_no, val) \
                                or val_in_box(matrix, row_no, col_no, val):
                            new_block.remove(val)
                            changed = True
                new_row.append(new_block)
        new_matrix.append(new_row)
    
    return new_matrix, changed

# Checks if the value being looked for can be found in the specified row
def val_in_row(matrix, row_no, val):
    if [val] in matrix[row_no]:
        return True
    else:
        return False

# Checks if the value being looked for can be found in the specified column
def val_in_col(matrix, col_no, val):
    col_list = []
    for row in matrix:
        col_list.append(row[col_no])
    if [val] in col_list:
        return True
    else:
        return False

# Checks if the value being looked for can be found in the current 3x3 box
def val_in_box(matrix, row_no, col_no, val):
    box_row = row_no / 3
    box_col = col_no / 3

    box_vals = []
    for row in range(box_row * 3, box_row * 3 + 3):
        for col in range(box_col * 3, box_col * 3 + 3):
            box_vals.append(matrix[row][col])
    if [val] in box_vals:
        return True
    else:
        return False


# Checks if a sudoku game has been solved yet
def solved(matrix):
    for row in matrix:
        for block in row:
            if len(block) > 1:
                return False
    return True

# Checks to make sure that the matrix is still valid
def consistent(matrix):
    # Row check
    for row in matrix:
        for digit in range(1, 10):
            if row.count([digit]) > 1:
                return False
    # Column check
    for col_no in range(9):
        col_list = []
        for row in matrix:
            col_list.append(row[col_no])
        for digit in range(1, 10):
            if col_list.count([digit]) > 1:
                return False
    # Box check
    for box_row in range(3):
        for box_col in range(3):
            box_list = []
            for row in range(3):
                for col in range(3):
                    box_list.append(matrix[box_row * 3 + row][box_col * 3 + col])
            for digit in range(1, 10):
                if box_list.count([digit]) > 1:
                    return False
                
    return True


# Writes the solution into the output file in easy-to-read format
def write_solution(outfile, solution, count):
    outfile.write("Grid %s Solution\n" % string.zfill(count, 2))
    for row in solution:
        outfile.write('%s\n' %''.join(map(str, row)))

def main():

    output_filename = 'sudoku_solutions.txt'

    print "This program takes as input a file full of Sudoku puzzles, formatted like so:\n"
    print "Grid 01"
    print "003020600"
    print "900305001"
    print "001806400"
    print "008102900"
    print "700000008"
    print "006708200"
    print "002609500"
    print "800203009"
    print "005010300"
    print "Grid 02"
    print "200080300"
    print "060070084"
    print "030500209"
    print "000105408"
    print "000000000"
    print "402706000"
    print "301007040"
    print "720040060"
    print "004010003"
    print " . . . \n"
    print "In the above puzzles, '0' represents a blank that must be filled.\n"
    filename = raw_input("Please specify a filename to open: ").strip()
    infile = open(filename, 'r')
    outfile = open(output_filename, 'w')
    games = parse_games(infile)
    infile.close()

    count = 1
    for game in games:
        print "Solving puzzle %s..." % count
        solution = solve_game(game)
        if solution == 0:
            outfile.write("Puzzle %s was too hard.\n\n" % count)
        else:
            write_solution(outfile, solution, count)
        count += 1
    print "Solving complete! Check %s for solutions." % output_filename

if __name__ == '__main__':
    main()